##! 
##! Coptright(c) 2023 Stanford Research Systems, All right reserved
##! Subject to the MIT License
##! 

import time
from srsinst.uga import Keys

from srsgui import Task
from srsgui.task.inputs import InstrumentInput, ListInput, IntegerInput
from srsinst.uga import get_uga


class UGAModeControlTask(Task):
    """
    update
    """
    InstrumentName = 'uga to control'
    ModeName = 'mode control'
    UpdatePeriod = 'update period'
    LeakTestMass = 'mass for leak test'
    ModeList = [Keys.Start, Keys.Stop, Keys.Sleep, Keys.LeakTestOn, Keys.LeakTestOff,
                Keys.SystemBakeOn, Keys.SystemBakeOff]

    input_parameters = {
        InstrumentName: InstrumentInput(),
        ModeName: ListInput(ModeList),
        LeakTestMass: IntegerInput(4, ' AMU', 1, 100),
        UpdatePeriod: IntegerInput(2, ' s', 1, 60, 1)
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()

        self.uga = get_uga(self, self.params[self.InstrumentName])
        print(self.uga.status.id_string)
        while self.uga.status.error != Keys.OK:
            pass

        self.immediate_state = self.uga.mode.state
        self.final_state = self.immediate_state
        self.logger.info('Current mode before changing: {}'.format(self.immediate_state))

    def test(self):
        if self.params[self.ModeName] == Keys.Start:
            self.uga.mode.start()
            self.immediate_state = Keys.Start
            self.final_state = Keys.Ready

        elif self.params[self.ModeName] == Keys.Stop:
            self.uga.mode.stop()
            self.immediate_state = Keys.Stop
            self.final_state = Keys.Off

        elif self.params[self.ModeName] == Keys.Sleep:
            self.uga.mode.sleep()
            self.immediate_state = Keys.Sleep
            self.final_state = Keys.Idle

        elif self.params[self.ModeName] == Keys.LeakTestOn:
            if self.uga.mode.state != Keys.Ready:
                raise ValueError('Leak test mode is available only from READY state')
            self.uga.mode.leak_test_mass = self.params[self.LeakTestMass]
            self.uga.mode.leak_test = True
            self.immediate_state = Keys.Ready
            self.final_state = Keys.LeakTest

        elif self.params[self.ModeName] == Keys.LeakTestOff:
            if self.uga.mode.state != Keys.LeakTest:
                raise ValueError('Leak test mode is not on')

            self.uga.mode.leak_test = False
            self.immediate_state = Keys.LeakTest
            self.final_state = Keys.Ready

        elif self.params[self.ModeName] == Keys.SystemBakeOn:
            self.immediate_state = self.uga.mode.state
            self.uga.mode.bake = True
            self.final_state = Keys.SystemBake

        elif self.params[self.ModeName] == Keys.SystemBakeOff:
            if self.uga.mode.state != Keys.SystemBake:
                raise ValueError('SYSTEM BAKE is not on')
            self.uga.mode.bake = False
            self.immediate_state = Keys.Start
            self.final_state = Keys.Ready

        error_code = self.uga.status.error_number
        if error_code > 10:
            raise ValueError('Error "{}" when trying to change to  {}'
                             .format(self.uga.status.error_message[error_code], self.final_state))
        else:
            self.logger.info('UGA mode changing to {}'.format(self.final_state))

        self.display_device_info(device_name=self.params[self.InstrumentName], update=True)

        time.sleep(self.params[self.UpdatePeriod])
        current_state = self.uga.mode.state
        while current_state in [self.immediate_state, self.final_state]:
            if not self.is_running():
                break
            self.display_device_info(device_name=self.params[self.InstrumentName], update=True)
            current_state = self.uga.mode.state
            if current_state == self.final_state:
                self.logger.info('Mode successfully changed to {}'.format(self.final_state))
                break
            time.sleep(self.params[self.UpdatePeriod])

        self.display_device_info(device_name=self.params[self.InstrumentName], update=True)
        self.set_task_passed(True)

    def cleanup(self):
        pass



##! 
##! Coptright(c) 2023 Stanford Research Systems, All right reserved
##! Subject to the MIT License
##! 

from srsgui.inst.component import Component
from srsgui.inst.instrument import Instrument, SerialInterface, TcpipInterface
from srsgui.inst.exceptions import InstIdError

from srsgui.task.inputs import FindListInput, IntegerListInput, Ip4Input, StringInput, PasswordInput

from .components import BypassPump, RoughingPump, TurboPump, \
                        BypassValve, SampleValve, VentValve, \
                        RGA, IonGauge,  MultipleInlet, \
                        Pressure, Heaters, Temperature, \
                        Ethernet, Status, Mode
from .keys import Keys


class UGA100(Instrument):
    _IdString = 'SRS_UGA'
    _term_char = b'\r'

    available_interfaces = [
        [   SerialInterface,
            {
                'port': FindListInput(),
                'baud_rate': IntegerListInput([28800, 38400]),
                'hardware_flow_control': True
            }
        ],
        [   TcpipInterface,
            {
                'ip_address': Ip4Input('192.168.1.10'),
                'user_id': StringInput('srsuga'),
                'password': PasswordInput('srsuga'),
                'port': 818
            }
        ]
    ]

    def __init__(self, interface_type=None, *args):
        super().__init__(interface_type, *args)

        self.mode = Mode(self)
        self.bp = BypassPump(self)
        self.rp = RoughingPump(self)
        self.tp = TurboPump(self)
        self.bv = BypassValve(self)
        self.sv = SampleValve(self)
        self.vv = VentValve(self)
        self.rga = RGA(self)
        self.ig = IonGauge(self)
        self.mi = MultipleInlet(self)
        self.ht = Heaters(self)

        self.temperature = Temperature(self)
        self.pressure = Pressure(self)
        self.ethernet = Ethernet(self)
        self.status = Status(self)

    def check_id(self):
        if not self.is_connected():
            return None, None, None

        reply = self.query_text('ZQID?').strip()
        strings = reply.split(',')

        if len(strings) != 3:
            return None, None, None

        model_name = strings[0].strip()
        serial_number = strings[1].strip()[4:]
        firmware_version = strings[2].strip()[2:]

        if self._IdString not in reply:
            raise InstIdError("Invalid instrument: {} not in {}"
                              .format(self._IdString, reply))
        if self.rga.state == Keys.On:
            self.rga.check_id()  # configure RGA

        if 'UGA_HT' in model_name:
            if hasattr(self, 'bv'):
                del self.bv
            if hasattr(self, 'sv'):
                del self.sv
            if hasattr(self, 'mi'):
                del self.mi
            if not hasattr(self, 'bp'):
                self.bp = BypassPump(self)
        elif 'UGA_LT' in model_name:
            if not hasattr(self, 'bv'):
                self.bv = BypassValve(self)
            if not hasattr(self, 'sv'):
                self.sv = SampleValve(self)
            if not hasattr(self, 'mi'):
                self.mi = MultipleInlet(self)
            if hasattr(self, 'bp'):
                del self.bp
        elif 'UGA_PM' in model_name:
            if hasattr(self, 'bv'):
                del self.bv
            if not hasattr(self, 'sv'):
                self.sv = SampleValve(self)
            if not hasattr(self, 'mi'):
                self.mi = MultipleInlet(self)
            if hasattr(self, 'bp'):
                del self.bp
        elif 'UGA' in model_name:
            if not hasattr(self, 'bv'):
                self.bv = BypassValve(self)
            if not hasattr(self, 'sv'):
                self.sv = SampleValve(self)
            if not hasattr(self, 'mi'):
                self.mi = MultipleInlet(self)
            if not hasattr(self, 'bp'):
                self.bp = BypassPump(self)

        self._id_string = reply
        self._model_name = model_name
        self._serial_number = serial_number
        self._firmware_version = firmware_version
        return self._model_name, self._serial_number, self._firmware_version
        
    def get_status(self):
        return self.status.get_status_text()

    def reset(self):
        self.comm.send('ZRST')

    def handle_command(self, cmd_string: str):
        cmd = cmd_string.upper()
        reply = ''
        if '?' in cmd or cmd.startswith("FL") or cmd.startswith("HV") or \
                cmd.startswith("VF") or cmd.startswith("EE") or \
                cmd.startswith("IE") or cmd.startswith("IN"):
            reply = self.query_text(cmd).strip()
        elif cmd.startswith("MR"):
            # self.send(cmd)
            try:
                mass = int(cmd[2:].strip())
                intensity = self.rga.scan.get_single_mass_scan(mass)  # read_long()
                reply = str(intensity)
            except:
                pass
        elif cmd.startswith("SC") and len(cmd) < 10:
            self.rga.scan.get_analog_scan()
            reply = "Scan Completed"
        elif cmd.startswith("HS"):
            self.rga.scan.get_histogram_scan()
            reply = "Scan Completed"
        else:
            self.send(cmd)
        return reply


if __name__ == '__main__':
    uga = UGA100('tcpip', '172.25.128.13', 'srsuga', 'srsuga')
    print(uga.status.id_string)

